/*
 * Pebble_LDEF.c
 *
 * A LDEF for the Pebbles program.  The actual LDEF is part of a separate project.
 * The LDEF code resource merely calls a function stored in a List Item's RefNum; 
 * this function has the same arguments as the LDEF.  This trick allows one to keep
 * their LDEF in their program proper, with the following benefits: use of global data, 
 * calls to other functions, easy recompilation/debugging.
 *
 * This file contains the function called directly by the actual LDEF, as well as some
 * support routines.
 *
 * 26 FEB 95	MDT
 */

/* #includes */

#include "Pebble_LDEF.h"

/* #defines */

/* Set the following to true to check allignments.*/
#define kDrawLines	false

/* Text drawing offsets within our List Rect. If the size of the Rect changes, then
 * our constants must change too.
 */
 
#define kCheckStart	1
#define kNameStart	14
#define kVolRight	118
#define kLongRight	153 + 8
#define kIntRight	188 + 16
#define kShortRight	223 + 24


/* Global Data */

/* The following pointer points to a documented un-documented (read it again) system 
 * global variable: the variable that stores the system default selection color (set 
 * via the Color control panel.
 */
Ptr HiliteMode = (Ptr)0x938; 

/* Private Function Prototypes */

void MyFloatToString (float f, Str255 s);


/* Code */

/* MyList
 * This is our virtual LDEF.  Read note above; then go find the NewsWatcher source code
 * and steal for yourself.
 */
void MyList(short lMessage, Boolean lSelect, Rect *lRect, Cell lCell,
					short lDataOffset, short lDataLen, ListHandle lHandle)
{
	Ptr cellData;
	ListPtr theListPtr;
	SignedByte hStateList, hStateCells;
	FontInfo fontInfo;
	short	curFont;
	Str255	s;
	RgnHandle saveRgn = NewRgn();
	Rect	clipRect;

	PebbleListRecord pebble;
	
	// Lock down the handles
	hStateList = HGetState((Handle) lHandle);
	HLock((Handle) lHandle);
	theListPtr = *lHandle;
	hStateCells = HGetState((Handle) theListPtr->cells);
	HLock((Handle) theListPtr->cells);
	cellData = *(theListPtr->cells);
	
	GetFontInfo(&fontInfo);
	curFont = (*lHandle)->port->txFont;
	
	switch(lMessage) {
	case lDrawMsg:
		TextFont(1);	// Geneva
		EraseRect(lRect);
		GetClip(saveRgn);
		
		BlockMove( cellData + lDataOffset, &pebble, lDataLen);
		if (pebble.cookie != (char)13) {	/* Bad cookie */
			break;
		}
		
		
		if (kDrawLines) {	/* Compiled Constant */
		/* draw some lines for testing */
		MoveTo(lRect->left+ (kNameStart-2), lRect->top);	/* Between check and label */
		LineTo(lRect->left+ (kNameStart-2), lRect->bottom);
		MoveTo(lRect->left+ (kVolRight - 35-8), lRect->top);	/* Between label and v */
		LineTo(lRect->left+ (kVolRight - 35-8), lRect->bottom);
		MoveTo(lRect->left+ kVolRight, lRect->top);	/* Between v and l */
		LineTo(lRect->left+ kVolRight, lRect->bottom);
		MoveTo(lRect->left+ kLongRight, lRect->top);	/* Between l and i */
		LineTo(lRect->left+ kLongRight, lRect->bottom);
		MoveTo(lRect->left+ kIntRight, lRect->top);	/* Between i and s */
		LineTo(lRect->left+ kIntRight, lRect->bottom);
		}
		
		if (pebble.hasFile) {
			MoveTo(lRect->left + kCheckStart, 
				lRect->bottom - (fontInfo.descent + fontInfo.leading));
			DrawString("\p");	/* Put into Resource file? */
		}
		MoveTo(lRect->left + kNameStart, 
			lRect->bottom - (fontInfo.descent + fontInfo.leading));
		DrawString((unsigned char *)pebble.name); 

		MyFloatToString(pebble.v, s);
		SetRect(&clipRect, lRect->left + (kVolRight - 35-8), lRect->top, lRect->left + kVolRight, lRect->bottom);
		ClipRect(&clipRect);
		MoveTo(lRect->left + (kVolRight - StringWidth(s)), 
			lRect->bottom - (fontInfo.descent + fontInfo.leading));
		DrawString(s); 

		MyFloatToString(pebble.l, s);
		SetRect(&clipRect, lRect->left + (kLongRight - 35-8), lRect->top, lRect->left + kLongRight, lRect->bottom);
		ClipRect(&clipRect);
		MoveTo(lRect->left + (kLongRight - StringWidth(s)), 
			lRect->bottom - (fontInfo.descent + fontInfo.leading));
		DrawString(s); 

		MyFloatToString(pebble.i, s);
		SetRect(&clipRect, lRect->left + (kIntRight - 35-8), lRect->top, lRect->left + kIntRight, lRect->bottom);
		ClipRect(&clipRect);
		MoveTo(lRect->left + (kIntRight - StringWidth(s)), 
			lRect->bottom - (fontInfo.descent + fontInfo.leading));
		DrawString(s); 

		MyFloatToString(pebble.s, s);
		SetRect(&clipRect, lRect->left + (kShortRight - 35-8), lRect->top, lRect->left + kShortRight, lRect->bottom);
		ClipRect(&clipRect);
		MoveTo(lRect->left + (kShortRight - StringWidth(s)), 
			lRect->bottom - (fontInfo.descent + fontInfo.leading));
		DrawString(s); 
		
		SetClip(saveRgn);

		if (lSelect) {
			BitClr(HiliteMode, pHiliteBit);	/* THIS is how you hilite with the default color. */
	  		InvertRect(lRect);	
		} 

		break;
	case lHiliteMsg:
		BitClr(HiliteMode, pHiliteBit);
	  	InvertRect(lRect);	
		break;
	default:
		;
	}
	
	// Restore everything, please
	
	TextFont(curFont);
	HSetState((Handle) theListPtr->cells,hStateCells);
	HSetState((Handle) lHandle, hStateList);
}

/* MyFloatToString
 * This is a simple Float to String converter; it assumes one decimal place of accuracy.
 * There are lots of different ways to do this, but I'm doing this way!
 */
void MyFloatToString (float f, Str255 s)
{
	long 	l;
	Str255	d;
	
	f += 0.05; /* Force rounding up or down */
	
	l = f;
	NumToString(l, s);
	l = ((f - l) * 10); /* The next decimal place. */
	NumToString(l, d);
	d[2] = d[1]; d[1] = (unsigned char)'.'; d[0]++;
	BlockMove( &(d[1]) , &(s[s[0]+1]), d[0]);
	s[0] += d[0];
}
