/*
 * Pebble_ListItem.cp
 *
 * This is a ListItem class that manages the list item in the InputWindow.
 *
 * 24 FEB 95 MDT
 */

#include "Pebble_Alerts.h"
#include "Pebble_LDEF.h"
#include "Pebble_ListItem.h"
#include "Pebble_ResIDs.h"

#define		kDefaultList	0
#define		kDrawList		true
#define		kDontDrawList	false
#define		kNoGrowBox		false
#define		kNoHScroll		false
#define		kVScroll		true


/* TListItem::TListItem
 * TListItem initializes data, creates the List Item, and sets a few List Item 
 * fields.  The refCon points to our fake LDEF function: see Pebble_LDEF. 
 */ 
TListItem::TListItem(WindowPtr w,Rect *r)
{
	Rect	dataBounds, rView = *r;
	Point	cSize;
	
	SetRect(&dataBounds, 0,0,1,0);
	rView.right -= 15; // make room for scroll bars 
	
	SetPt(&cSize, (rView.right-rView.left),16 );
		
	listHandle = LNew(&rView, &dataBounds, cSize, kPebbleLDEFResID, w, kDontDrawList, 
					   kNoGrowBox, kNoHScroll, kVScroll);
	if (!listHandle) {
		DoError("\pFatal error: LNew. Try increasing program Memory.",0,kFatal);
	}
	(*listHandle)->selFlags = lOnlyOne;
	(*listHandle)->refCon = (long) MyList;		// THIS IS DAMN IMPORTANT!!!
	lastCellIndex = 0;
}

/* TListItem::~TListItem()
 * This destroys the list. See Inside Macintosh IV.  For those wimps with the
 * NIM volumes, go look it up yourself.
 */
TListItem::~TListItem()
{
	LDispose(listHandle);
}

/* TListItem::AddItem
 * AddItem adds an item to the List. This is fairly application/LDEF dependant.
 */
void TListItem::AddItem(PebbleListRecord *pebble)

{
	Cell	cell, theCell={0,0};
	
	lastCellIndex = LAddRow (1,lastCellIndex+1, listHandle);
	SetPt(&cell, 0, lastCellIndex);
	LSetCell(pebble,sizeof(PebbleListRecord),cell,listHandle); 
	
	this->Select(lastCellIndex);
}

/* TListItem::ChangeItem
 * ChangeItem changes the contents of an existing List Item item.
 */
void TListItem::ChangeItem(short v,PebbleListRecord *pebble)

{
	Cell	cell, theCell={0,0};
	
	SetPt(&cell, 0, v);
	LSetCell(pebble,sizeof(PebbleListRecord),cell,listHandle); 
	
	this->Select(v);
}

/* TListItem::Select
 * Select deselects the currently selected cell, and selects the specified cell.  If
 * v is negative, no cell is selected after the deselection.  If the selected item isn't
 * visible, the List is automatically scrolled to display the item. 
 */
void TListItem::Select(short v)
{
	Cell	cell, theCell={0,0};
	
	/* Un-Select Last Cell */
	if(LGetSelect(true, &theCell, listHandle)) 
		LSetSelect(false, theCell, listHandle);

	if (v >= 0) {
		/* Set Cell */
		SetPt(&cell, 0, v);
	
		/* Select Cell */
		LSetSelect(true, cell, listHandle);
		/* Make Item Visible */
		LAutoScroll(listHandle);
	}	
}

/* TListItem::RemoveItem
 * Provided for my benefit for other projects
 */
void TListItem::RemoveItem()
{
}

/* TListItem::Update
 * Update is called whenever the owning Window receives an Update event.  It draws the
 * list, including drawing selected items and whatnot.
 */
void TListItem::Update()
{
	LUpdate((*listHandle)->port->visRgn, listHandle);
}

/* TListItem::Click
 * Click is called whenever the owning Window receives a mouseDown event in its Content
 * region.  where is a point in Global Co-ordinates, taken directly from the event record.
 */
short TListItem::Click(Point where)
{
	Cell cell;
	GlobalToLocal(&where);
	if (LClick(where, 0, listHandle)) {
		cell = LLastClick(listHandle);
		return cell.v;
	}
	return -1;
}

/* TListItem::DrawingOff
 * This prevents the list from drawing itself. For more information about this, see
 * Inside Mac IV.
 */
void TListItem::DrawingOff()
{
	LDoDraw(false, listHandle);
}

/* TListItem::DrawingOn
 * This allows the list to draw itself. For more information about this, see
 * Inside Mac IV.
 */void TListItem::DrawingOn()
{
	LDoDraw(true, listHandle);
}