/*
 * Cursors.c
 */

#include "Cursors.h"
#include "MMR.h"
#include "Map Window.h"
#include "Eventsx.h"
#include "LinkedList.h"
#include "Alertsx.h"

MouseCursors gCursors;			//record of cursors.
//CursorTracking gCursTracking;
LinkedListPtr gCursWindowList;

/* InitMouseCursors
 * Load our cursor resources into our gCursors record.
 */
void InitMouseCursors(void)
{
	short i;
	
	gCursors.crsrID[0] = 0; // standard cursor... we don't store the graphics for it.
	
	for (i = 0; i < kNumCrsr; i++) {
		gCursors.crsrID[i+1] = i + kBaseCrsrID;
		gCursors.crsr[i+1] = GetCCursor(i + kBaseCrsrID);
	}
	
	// set up our gCursTracking structure.
	/*gCursTracking.ctRgn = nil;
	gCursTracking.status = off;
	gCursTracking.primaryID = 0;
	gCursTracking.secondaryID = 0;
	gCursTracking.watchPhase = 0;*/
	
	// 
	NewLinkedList(&gCursWindowList);
}

/* StartCursTracking
 * Sets gCursTracking info, as well as mouse tracking Rgn.
 * Doesn't actually set the mouse cursor. Remarkably, the Event Manager takes
 * care of all of that for us. Confused? Meditate on this code a while.
 */
/* Remember, StartCursTracking can be called repeatedly to change the cursor for
 * a given window. Therefore, the CursorTracking ptr will be created only if
 * necessary, forcing one entry per window.
 *
 * In the future, one should perhaps add a ChangeCursor method...
 * 2/18/96 MDT (happy birthday, Angela)
 */
void StartCursTracking(WindowPtr w, RgnHandle rgn, short primary, short secondary)
{
	CursorTracking *ct;
	Boolean newPtrWasCreated = false;
	
	// get ct info.
 	ct = (CursorTracking*)GetListItemByKey(gCursWindowList,(long)w);
	// if there is no ct info, make a new ptr.
	if (ct == nil) {			
		ct = (CursorTracking*) NewPtr(sizeof(CursorTracking));
		if (!ct) DoError("\pFailed to make CursorTracking record",MemError(),kFatal);
		newPtrWasCreated = true;
	}
	ct->ctRgn = rgn;
	ct->status = arrow;
	ct->primaryID = primary;
	ct->secondaryID = secondary;
	ct->watchPhase = 0;

	// only add ct to our gCursWindowList if necessary
	if (newPtrWasCreated) AddItemToList(gCursWindowList, (long)ct, (long)w);
	
	if (gEMTracking.mouseRgn == nil) gEMTracking.mouseRgn = NewRgn();
	CopyRgn(rgn,gEMTracking.mouseRgn);
	
	SetProperMouseCursor(w);
}

void EndCursTracking(WindowPtr w)
{
	CursorTracking *ct;

	if (!w) return; 
	ct = (CursorTracking*)GetListItemByKey(gCursWindowList,(long)w);
	if (ct == nil) return;
	
	if (gEMTracking.mouseRgn) DisposeRgn(gEMTracking.mouseRgn);
	gEMTracking.mouseRgn = nil;
	
	RemoveItemFromList(gCursWindowList,(long)w);
	
	if (ct->ctRgn) DisposeRgn(ct->ctRgn);
	/*gCursorTracking.ctRgn = nil;
	gCursorTracking.status = off;
	gCursorTracking.primaryID = 0;
	gCursorTracking.secondaryID = 0;*/
	DisposePtr((Ptr)ct);
	
	SetMouseCursor(kCrsrArrow);
}

void SuspendCursTracking(void)
{
	if (gEMTracking.mouseRgn) DisposeRgn(gEMTracking.mouseRgn);
	gEMTracking.mouseRgn = nil;
	SetMouseCursor(kCrsrArrow);
}

void ResumeCursTracking(WindowPtr w)
{
	CursorTracking *ct;

	if (!w) return; 
	ct = (CursorTracking*)GetListItemByKey(gCursWindowList,(long)w);
	if (ct == nil) return;
	
	if (ct->ctRgn == nil) return; // if we're not really tracking, then don't bother.
	
	if (gEMTracking.mouseRgn == nil) gEMTracking.mouseRgn = NewRgn();
	CopyRgn(ct->ctRgn,gEMTracking.mouseRgn);
	
	SetProperMouseCursor(w);
}

/* DoMouseMoved
 * If a cursor moves (or simply is) outside a mouse region specified by StartCursorTracking, 
 * DoMouseMoved is called. DoMouseMoved inverts the region so that DoMouseMoved is called 
 * when the mouse is moved INTO the region again.  The next time it is called, it un-inverts
 * the region and so on....
 */ 
/* PREREQ: PORT MUST BE SET!! - this can change once we keep track of windows...
 */
void DoMouseMoved(WindowPtr w, Boolean optionKeyIsDown)
{	
	CursorTracking *ct;

	if (!w) return; 
	ct = (CursorTracking*)GetListItemByKey(gCursWindowList,(long)w);
	if (ct == nil) return;
	
	if (ct->ctRgn == nil) return; // if we're not really tracking, then don't bother.

	// set proper cursor
	SetProperMouseCursor(w);
	
	// set proper rgn
	OpenRgn();
	if (ct->status == arrow) FrameRect(&qd.screenBits.bounds); // not multi-monitor safe?
	FrameRgn(ct->ctRgn);
	CloseRgn(gEMTracking.mouseRgn);
}

/* CursorEventProcessing
 * Called by the Event loop to determine if we need to switch between primary and
 * secondary modes
 */
void CursorEventProcessing(WindowPtr w, Boolean optionKeyIsDown)
{
	CursorTracking *ct;

	if (!w) return; 
	ct = (CursorTracking*)GetListItemByKey(gCursWindowList,(long)w);
	if (ct == nil) return;
	
	if (ct->ctRgn == nil) return; // if we're not really tracking, then don't bother.
	if ((ct->status != secondary) && (ct->status != primary)) return;
	
	if (optionKeyIsDown) {
		if (ct->status != secondary) SetMouseCursor(ct->secondaryID);
		ct->status = secondary;
	} else {
		if (ct->status != primary) SetMouseCursor(ct->primaryID);
		ct->status = primary;
	}
}

void DoWatchCursor(WindowPtr w, short action)
{
	CursorTracking *ct;

	if (!w) return; 
	ct = (CursorTracking*)GetListItemByKey(gCursWindowList,(long)w);
	if (ct == nil) return;
	
	switch (action){
	case kCrsrWatchOn: // activate
		ct->watchPhase = 0;
		ct->status = watch;
		SetMouseCursor(kCrsrWatch);
		break;
	case kCrsrWatchOff: // deactivate
		ct->watchPhase = 0;
		// for lack of a better idea, assume arrow cursor.
		ct->status = arrow;
		SetMouseCursor(kCrsrArrow);
		break;
	case kCrsrWatchSpin: // spin
		ct->status = watch;
		SetMouseCursor(kCrsrWatch + ct->watchPhase);
		ct->watchPhase++;
		if (ct->watchPhase == kNumCrsrWatches) ct->watchPhase = 0;
		break;
	default:
		;
	}
}

void SetMouseCursor(short crsrType)
{
	CCrsrHandle crsr;
	
	if ((crsrType < 1) || (crsrType > kNumCrsr)) {
		FixCursor();
	} else {
		crsr = 	gCursors.crsr[crsrType];
		SetCCursor(crsr);
	}
}

/* SetProperMouseCursor
 * (I couldn't think of a better name, so here's a good comment.)
 * This routine looks at the position of the mouse cursor, checks that point against
 * the gCursorTracking.ctRgn, and sets the cursor appropriately.
 */
/* For the future: might want to think about replacing GetMouse with mouse coordinates
 * passed in from the Event Record.
 */
void SetProperMouseCursor(WindowPtr w)
{
	Point mousePt;
	CursorTracking *ct;

	if (!w) return; 
	ct = (CursorTracking*)GetListItemByKey(gCursWindowList,(long)w);
	if (ct == nil) return;
	
	GetMouse(&mousePt);
	LocalToGlobal(&mousePt);
	
	if (PtInRgn(mousePt, ct->ctRgn)) {
		if (IsOptionKeyDown()) {
			SetMouseCursor(ct->secondaryID);
			ct->status = secondary;
		} else {
			SetMouseCursor(ct->primaryID);
			ct->status = primary;
		}
	} else {
		SetMouseCursor(kCrsrArrow);
		ct->status = arrow;
	}
}