/*
 * DLOGUtil.h
 *
 * Dialog Utilities - Some routines useful in Dialog and Alert EventProcs.
 *
 */
// GetDItem2 and SetDItem2 - 5/21/96
#include "DLOGUtil.h"
#include "MMR.h"

#define kEnterKey		3
#define kBackspace		8
#define kTab			9
#define kReturnKey		13
#define kEscapeKey		27
#define kLeftKey		28
#define kRightKey		29
#define kUpKey			30
#define kDownKey		31
#define kDelete			0xFF


// oh no! software might break!
//StringHandle	TEScrpHandle = (StringHandle)0x0AB4;


/* IsDlgNumber
 * Determines if a character is valid input in a number-only dialog item.
 * In addition to verifying the number falls in the 0-9 range, it also
 * allows certain control characters; i.e., Return, Tab, arrows, etc.
 */
Boolean IsDlgNumber(char c)
{
	if (IsDlgControl(c) || IsNumber(c)) return true;
	
	return false;
}

Boolean IsDlgControl(char c)
{
	if ((c >= kEscapeKey) && (c <= kDownKey)) return true;
	if ((c == kReturnKey) || (c == kEnterKey) || (c == kDelete) 
		|| (c == kBackspace) || (c == kTab)) return true; 
	
	return false;
}

Boolean IsNumber(char c)
{
	if ((c >= '0') && (c<='9')) return true;
	
	return false;
}

Boolean IsPrintableChar(char c)
{
	if ((c > 32) && (c != 0xFF)) return true;
	return false;
}

/* MakeDLOGString
 * Given a DialogPtr, EventPtr and an item number, it returns what the string 
 * will be if the current event is processed. Should only be called from the 
 * dialog's event proc. If the input charater is a control character, the
 * routine will not alter the item's string.
 *
 * Oh ... I meant to say, it should only be called on TextEdit DLOG items.
 * 8/20/95
 */

void MakeDLOGString (DialogPtr d, EventRecord *e, short item, Str255 str)
{
	short			*TEScrpLength = (short *)0x0AB0;
	DialogRecord	*dr;
	TEHandle		teh;
	char			c;
	Str255			input, text;
	short 			selStart, // the character after which the ins. pt. lies
					selEnd;   // the character before which the ins. pt. lies
	short 			i;
	short			strIdx=0;
	
	short			iType;
	Handle			iHandle;
	Rect			iRect;
	
	// get the text string
	GetDItem(d,item,&iType,&iHandle,&iRect);
	GetIText(iHandle,text);

	// Set the input string
	c = (e->message & charCodeMask);
	// if it's a control character, return the dlog item text.
	if (IsDlgControl(c)) {
		PStringCopy(str,text);
		return;
	} else if (e->modifiers & cmdKey) {
		if  ((c == 'v') || (c == 'V')) {
			// if pasting ... get the pasted string.
			(void)TEFromScrap();
			HLock(iHandle);
			iHandle = TEScrapHandle();
			for (i=0; i< *TEScrpLength; i++) {
				input[i+1]=((unsigned char *)(*iHandle))[i];
			}
			input[0]=*TEScrpLength;
			HUnlock(iHandle);
		} else { // if any other command stroke 
			PStringCopy(str,text);
			return;
		}
	} else {
		// else, set the input string equal to the new character
		input[0]=1;
		input[1]=(unsigned char) c;
	}
	 
	// get the selection point in the TERec
	dr = (DialogRecord *)d;
	teh = dr->textH;
	HLock((Handle)teh);
	selStart=(*teh)->selStart;
	selEnd=(*teh)->selEnd;
	HUnlock((Handle)teh);

	// generate output (str) string:
	//   copy the first bit of text:
	for (i=1; i<=selStart; i++) { 
		str[++strIdx]=text[i];
	}
	//   copy the input string:
	for (i=1; i<=input[0]; i++) {
		str[++strIdx]=input[i];
	}
	//   copy the last part of text:
	for (i=selEnd+1; i<=text[0]; i++) {
		str[++strIdx]=text[i];
	}
	//   lastly, set the length
	str[0] = strIdx;
}

/* AbortEvent
 * To be called from within a DLOG or ALRT EventProc. Beeps, sets event
 * to nullEvent, and returns false, thereby cancelling the event completely.
 */
Boolean	AbortEvent(EventRecord *e,Boolean beep)
{
	if (beep) SysBeep(3);
	e->what = nullEvent; 	// abort event
	return (false);
}

/* GetDItem2
 * An improved GetDItem, to be used in place of GetDItem. Much like a Unix call, if
 * any of the last three parameter is nil, it simply isn't returned on to the program.
 * This way, if the programmer is only interested in the handle, they only need 
 * create a Handle variable, not all three.
 */
GetDItem2(DialogPtr d,short i, short *t,Handle *h,Rect *r)
{
	short iType;
	Handle iHandle;
	Rect iRect;
	
	if (!d) return;
	if (i<1) return;
	
	GetDItem(d,i,&iType,&iHandle,&iRect);
	if (t) *t=iType;
	if (h) *h=iHandle;
	if (r) *r=iRect;
}

/* SetDItem2
 * Like, GetDItem2, this is a Unix-ified version of SetDItem. If any of the last three
 * parameters are nil, they will not be used in the subsequent SetDItem.
 * 
 * NOTE: Unlike SetDItem, all three of the final parameters are being passed by reference;
 * the original SetDItem passes only the Rect by reference.
 */
SetDItem2(DialogPtr d,short i, short *t,Handle *h,Rect *r)
{
	short iType;
	Handle iHandle;
	Rect iRect;
	
	if (!d) return;
	if (i<1) return;

	GetDItem(d,i,&iType,&iHandle,&iRect);
	if (t) iType = *t;
	if (h) iHandle=*h;
	if (r) iRect=*r;
	SetDItem(d,i,iType,iHandle,&iRect);
}