/*
 * Events MWFF.c
 */

#include "Eventsx.h"
#include "MMR.h"
#include "Alertsx.h"
#include "Map Window.h"
#include "Menusx.h"
#include "Scen Window.h"
#include "Scenario.h"
#include "TileMenuCommon.h"
#include "Maps.h"
#include "Printingx.h"
#include "Cursors30.h"
#include "About.h"
#include "WinManager.h"
#include "WindowUtil.h"

#define kRevertDLOGID	131

/* private ... */
void DoEvents(EventRecord *e);
void HandleMouseDown(EventRecord *eventPtr);
void HandleKeyDown (EventRecord *eventPtr);
void HandleMenuChoice (long menuChoice);
void HandleAppleMenu(short item);
void HandleFileMenu(short item);
void DoRevert(void);
void HandleEditMenu(short item);
void HandleMapsMenu(short item);
void HandleWindowMenu(short item);
void HandleScenFilesMenu(short item);
void DoDiskEvent (EventRecord *eventPtr);

Boolean ExcludeNonOutdoorMaps(WindowPtr w);

/* globals */
Boolean gDone = false;
Boolean gForeGround = true;
//EventMouseTracking gEMTracking;

void EventLoop(void)
{
	long sleep;
	EventRecord event;
	
	//gEMTracking.mouseRgn = nil;
		
	gScenario.file = nil;

	//InitMouseCursors();
	// UpdateFileMenu(); // must be called from open appleevents
	
	while (!gDone) {
		if (gForeGround) sleep = 1L;
		else sleep = 0x7FFFFFFF; 	// give a hoot!
		
		/*if (WaitNextEvent(everyEvent, &event, sleep, gEMTracking.mouseRgn)) {
			//if ((gEMTracking.mouseRgn) && (gForeGround)) 
				//CursorEventProcessing(FrontWindow(),(Boolean)(!!(event.modifiers & optionKey)));
			DoEvents(&event);
		}*/
		if (WaitNextEvent(everyEvent, &event, sleep, CCMouseRgn())) DoEvents(&event);
		CCEventProcessing();
	}
}

void DoEvents(EventRecord *e)
{
	WindowPtr w;
	static Boolean ctSuspended = false;
	
	switch (e->what) {
	case kHighLevelEvent:
		(void) AEProcessAppleEvent(e);
		break;
	case mouseDown:
		HandleMouseDown(e);
		break;
	case keyDown:
	case autoKey:
		HandleKeyDown(e);
		break;
	case diskEvt:							/* A disk has been inserted */
		DoDiskEvent(e);
		break;

	case activateEvt:						/* A window has been activated */
		w = (WindowPtr)e->message;
		if (FindWindowType(w) == kOutdoorsMap) {
			if (e->modifiers & 1) { // map window is activated
				MapActivate(w,true);
				//ResumeCursTracking(w);
				CCTrackCursor(w,true);
				EnableItem(GetMenu(mWindow), iZoomWindow); // kludge
			} else {
				MapActivate(w,false);
				//SuspendCursTracking();
				CCTrackCursor(w,false);
				DisableItem(GetMenu(mWindow), iZoomWindow); // kludge
			}		
		} else if (w == gScenWindow) ScenActivate(e->modifiers & 1);
		break;

	case updateEvt:
		w = (WindowPtr)e->message;
		if (w == gScenWindow) ScenUpdate();
		if (FindWindowType(w) == kOutdoorsMap) MapUpdate(w);
		
		break;


	case osEvt:								
		if (e->message & 0x01000000 ) {	// suspend or resume
			if ((e->message & suspendResumeMessage) == resumeFlag) {
				// Resume Event 
				gForeGround = true;
				w = FrontWindow();

				if (ctSuspended) {
					//ResumeCursTracking(FrontWindow());	
					CCTrackCursor(w,true);
					ctSuspended = false;	
				}						
				//else FixCursor();
				
				if (FindWindowType(w) == kOutdoorsMap) MapActivate(w,true);		
				else if (w == gScenWindow) ScenActivate(true);					
			}else {
				// Suspend Event 
				gForeGround = false;
				w = FrontWindow();
				//if (gEMTracking.mouseRgn) { // for now.
					//SuspendCursTracking();
					CCTrackCursor(w,false);
					ctSuspended = true;
				//} else {
					FixCursor();
				//}
				if (FindWindowType(w) == kOutdoorsMap) MapActivate(w,false);
				else if (w == gScenWindow) ScenActivate(false);					
			}
		}
		if ((e->message & 0xFF000000)==0xFA000000){ // cursor moved
			if (/*(gEMTracking.mouseRgn) &&*/ (gForeGround)) {
				SetPort(FrontWindow()); // this needs to be changed?	
				//DoMouseMoved(FrontWindow(),(Boolean)(!!(e->modifiers & optionKey)));
				CCMouseMoved();
			}
		}
		break;
	default:
		; 
	}
}

void HandleMouseDown(EventRecord *eventPtr)
{
	WindowPtr		whichWindow;
	short			thePart;
	long			menuChoice;
	Rect			limitRect;
	
	thePart = FindWindow(eventPtr->where, &whichWindow);
	
	switch (thePart) {
	
	case inMenuBar:
		HandleMenuChoice(MenuSelect (eventPtr->where));
		break;
		
	case inSysWindow:
		SystemClick(eventPtr, whichWindow);
		break;

	case inContent:
		if ((whichWindow != FrontWindow()) && (!(eventPtr->modifiers & cmdKey))) {
			SelectWindow(whichWindow);
		} else {
			if (whichWindow == gScenWindow) HandleScenContentClick(eventPtr);
			if (FindWindowType(whichWindow) == kOutdoorsMap) HandleMapContentClick(whichWindow,eventPtr);
		}
		break;

	case inGoAway:
		if (TrackGoAway(whichWindow,eventPtr->where)) {
			CloseMyWindow(whichWindow);
		}
		break;

	case inDrag:
		limitRect = qd.screenBits.bounds; 	// set limit to size of screen ...
		InsetRect(&limitRect,5,5);			// ... slightly inset.
		if (FindWindowType(whichWindow) == kOutdoorsMap) {
		 	MoveMapWindow(whichWindow, eventPtr, &limitRect);
		} else DragWindow(whichWindow, eventPtr->where, &limitRect);
		break;
	
	case inGrow:
		if (whichWindow == gScenWindow) ResizeScenWindow(eventPtr);
		if (FindWindowType(whichWindow) == kOutdoorsMap) ResizeMapWindow(whichWindow,eventPtr);
 		break;
	
	case inZoomIn:
	case inZoomOut:
		if (TrackBox(whichWindow,eventPtr->where,thePart)) {
			SetPort(whichWindow);
			EraseRect(&whichWindow->portRect);
			ZoomWindow(whichWindow,thePart,false);
			AutoSizeMapWindow(whichWindow);
			InvalRect(&whichWindow->portRect);
		}
		break;
		
	default:
		;
	}
}

void HandleKeyDown (EventRecord *eventPtr)
{
	char theChar;
	WindowPtr w = FrontWindow();
	long type;
	
	theChar = eventPtr->message & charCodeMask;

	// MDT 1/31/96 turn balloon help on/off
	if (theChar == 0x05) {
		(void) HMSetBalloons(!HMGetBalloons()); 
		return;
	}
		

	if ((eventPtr->modifiers & cmdKey) != 0)  { // if menu
		HandleMenuChoice(MenuKey (theChar));
		
		return;
	} 
	
	if (w != nil) {
		type = FindWindowType(w);
		if (type == kOutdoorsMap) {
			HandleMapKey(w,eventPtr, theChar);
		//} else if (type == kScenarioWindow)  {
		} else if (w == gScenWindow) { // register this with the WM in the future MDT
			HandleScenarioKey(eventPtr, theChar);
		}
	}
}

void HandleMenuChoice (long menuChoice)
{
	short 		menu;
	short		item;

	if (menuChoice != 0) {
		menu = HiWord(menuChoice);
		item = LoWord(menuChoice);
		
		switch (menu){
		case mApple:
			HandleAppleMenu(item);
			break;
		case mFile:
			HandleFileMenu(item);
			break;
		case mEdit:
			HandleEditMenu(item);
			break;
		case mMaps:
			HandleMapsMenu(item);
			break;
		case mWindow:
			HandleWindowMenu(item);
			break;
		case mScenFiles:
			HandleScenFilesMenu(item);
			break;
		default:
			;
		}
	}
	HiliteMenu (0); // no matter what menuChoice is 
}

/* HandleAppleMenu
 * This handles all Apple Menu selections
 */
void HandleAppleMenu(short item)
{
	MenuHandle	appleMenu;
	Str255		accName;
	short		accNumber;
	EventRecord	theEvent;
	
	switch (item){
	case iAbout:
		OpenAbout();
		for (;;) {
			if(WaitNextEvent(mouseDown+mouseUp+keyDown+autoKey, &theEvent,0,nil)) break;
		}	
		CloseAbout();
		break;
	default:
		appleMenu = GetMHandle (mApple);
		GetItem (appleMenu, item, accName);
		accNumber = OpenDeskAcc(accName);
		break;
	}
}

void HandleFileMenu(short item)
{
	WindowPtr	w;
	FSSpec spec;
	static Boolean opt;
	MenuHandle menu;
	
	switch (item) {
	case iNew:
		break;
	case iOpen:
		opt = IsOptionKeyDown();
		if (gScenWindow) break;
		if (!SelectScenarioFile(&spec)) break;
		(void) OpenScenarioFile(&spec,opt);
		SetTMTiles(gScenario.tiles);
		CheckTMItem(1);
		w = NewScenWindow();
		
		break;
	case iClose:
		w = FrontWindow();
		if (!w) break;
		CloseMyWindow(w);
		break;
	case iSave:
		if (gScenario.file) SaveScenarioFile(gScenario.file);
		break;
	case iSaveAs:
		(void) SaveNewScenarioFile(false);
		break;
	case iSaveCopy:
		(void) SaveNewScenarioFile(true);
		break;
	case iRevert:
		DoRevert();
		break;
	case iInstall:
		(void)InstallScenarioFile();
		break;
	case iExtract:
		(void)ExtractScenarioFile();
		SetTMTiles(gScenario.tiles);
		CheckTMItem(1);
		w = NewScenWindow();
		break;
	case iPageSetup:
		DoPageSetup();
		break;
	case iQuit:
		if (gScenWindow) if (!CloseScenWindow()) break;	
		gDone = true;
		break;
	default:
		;
	}
	UpdateFileMenu();
}

void HandleEditMenu(short item)
{
	switch (item) {
	default:
		;
	}
}

void HandleMapsMenu(short item)
{
}

void HandleWindowMenu(short item)
{
	WindowPtr	w;
	WindowPeek	wPeek;
	WStateData	*wsdp;
	short		part;
	
	switch(item) {
	case iTile:
		TileWindows(kTileHorizontal,0,AutoSizeMapWindow,ExcludeNonOutdoorMaps); // improve constants?
		break;
	case iTileVert:
		TileWindows(kTileVertical,0,AutoSizeMapWindow,ExcludeNonOutdoorMaps); // improve constants?
		break;
	case iStack:
		StackWindows(AutoSizeMapWindow,ExcludeNonOutdoorMaps);
		break;
	case iZoomWindow:
		w = FrontWindow(); // Pre: This is a zoomable window
		wPeek = (WindowPeek)w;
		wsdp = (WStateData*) *(wPeek->dataHandle);
		//if (EqualRect(&wsdp->stdState,&w->portRect)) part = inZoomIn;
		if (EqualRect(&wsdp->stdState,&(*(wPeek->contRgn))->rgnBBox) ) part = inZoomIn;
		else part = inZoomOut;

		SetPort(w);
		EraseRect(&w->portRect);
		ZoomWindow(w,part,false);
		AutoSizeMapWindow(w);
		//DrawMapWindow(w);
		InvalRect(&w->portRect);
		break;
			
	case iScenWindow:
		if (gScenWindow) SelectWindow(gScenWindow);
		break;
	default:
		if (item >= iWindow0) SelectWindow(FindNthWindow((item-iWindow0)+1));
	}
}

void HandleScenFilesMenu(short item)
{
	if (gScenWindow) CloseMyWindow(gScenWindow);
	(void)OpenScenarioFromList(item);
}


void DoRevert(void)
{
	DialogPtr d;
	ModalFilterUPP upp;
	short itemHit;
	GrafPtr save;
	//WindowPtr w = FrontWindow();

	GetPort(&save);

	d = GetNewDialog(kRevertDLOGID,nil,(WindowPtr)-1L);
	if (!d) return;
	ShowWindow(d);
	SetPort(d);
	(void) SetDialogDefaultItem(d,1);
	(void) SetDialogCancelItem(d,2);
	
	upp = NewModalFilterProc(AlertKeyProc);
	ModalDialog(upp,&itemHit);
	DisposeRoutineDescriptor(upp);
	
	SetPort(save);
	DisposeDialog(d);	
	
	if (itemHit == 1) {
		RevertAll();
		//if (FindWindowType(w) == kOutdoorsMap) CalcAndDrawMapWindow(w);
		ForEachMapWindowDo(CalcAndDrawMapWindow);
	}
}


/* DoDiskEvent
 * This is probably obsolete ...
 */
#define dlgTop		75                                   
#define dlgLeft		100

void DoDiskEvent (EventRecord *eventPtr)
{
	OSErr			ignore;
	Point			where;

	if (HiWord (eventPtr->message) != noErr) {
		DILoad();
		SetPt (&where, dlgLeft, dlgTop); 
		ignore = DIBadMount (where, eventPtr->message);
		DIUnload ();
	}
}

void CloseMyWindow(WindowPtr w)
{
	if (w == gScenWindow) {
		if (CloseScenWindow()) 	ForEachMapWindowDo(CloseMapWindow); 
		//CloseAllMapWindows(); // prevent problems!!
	}
	if (FindWindowType(w) == kOutdoorsMap) {
		CloseMapWindow(w);
		if (FindWindowType(FrontWindow()) == kOutdoorsMap) {
			EnableItem(GetMenu(mWindow), iZoomWindow); // kludge
		} else {
			DisableItem(GetMenu(mWindow), iZoomWindow); // kludge
		}
	}
}

Boolean ExcludeNonOutdoorMaps(WindowPtr w)
{
	if (FindWindowType(w) == kOutdoorsMap) return false;
	return true;
}