/*
 * LinkedList.c
 *
 * An all purpose linked list I can live with. Needs error checking.
 * 1/8/96
 */

#include "LinkedList.h"

/* NewLinkedList
 * Creates a new LinkedListMaster node, sets the lList to nil,
 * and sets the numEntries count to zero.
 * If NewLinkedList failed, *list will == nil.
 */
void NewLinkedList(LinkedListPtr *list)
{
	*list = (LinkedListPtr) NewPtr(sizeof(LinkedListMaster));
	if (*list)  { 
		(*list)->lList = nil;
		(*list)->numEntries = 0;
	}
}

/* DisposeLinkedList
 * Frees the internal linked list, and then frees the master node.
 * Becareful when storing pointers in here - you may orphan data!!
 */
void DisposeLinkedList(LinkedListPtr list)
{
	ListEntryPtr del;
	
	if (!list) return;
	
	while (list->lList != nil) {
		del = list->lList;
		list->lList = list->lList->next;
		DisposePtr((Ptr)del);
	}
	DisposePtr((Ptr)list);
}

/* AddItemToList
 * Adds an Item to the list. 
 */
void AddItemToList(LinkedListPtr list, long data, long key)
{
	ListEntryPtr item,cur;
	
	if (!list) return;

	item = (ListEntryPtr)NewPtr(sizeof(ListEntry));
	if (!item) return;
	item->key = key;
	item->data = data;
	
	cur = list->lList;
	if (cur == nil) {
		list->lList = item;
		item->next = nil;
	} else {
		while (cur->next != nil) cur=cur->next;
		item->next = cur->next;
		cur->next = item;
	}

	list->numEntries++;
}

void RemoveItemFromList(LinkedListPtr list, long key)
{
	ListEntryPtr cur,last;
	long ret;
	
	if (!list) return;

	cur = list->lList;
	last = nil;
	
	while ( (cur->key != key) && (cur != nil) ) {
		last = cur;
		cur = cur->next;
	}

	if (cur == nil) return;
	if (cur == list->lList) list->lList = list->lList->next; // also last = nil here
	else last->next = last->next->next;

	DisposePtr((Ptr)cur);	
	list->numEntries--;
}


long GetListItemByRank(LinkedListPtr list, long rank)
{
	short i;
	ListEntryPtr cur;
	
	if (!list) return 0;
	if ((rank < 1) || (rank > list->numEntries)) return 0;
	
	i=1; cur = list->lList; // i=1 not 0 *MDT 2/14/96
	while ((i != rank) && (cur != 0)) {
		cur = cur->next;
		i++;
	}
	
	return cur->data;
}

/*long GetListItemByData(LinkedListPtr list, long data);
*/

/* GetListItemByKey
 * Returns the data associated with a given key
 */
long GetListItemByKey(LinkedListPtr list, long key)
{
	ListEntryPtr cur;
	long ret;
	
	if (!list) return 0;

	cur = list->lList;
	
	while ( (cur->key != key) && (cur != nil) ) cur = cur->next;
	
	if (cur == nil) ret = 0;
	else ret = cur->data;
	
	return ret;
}

/* GetListKeyByRank
 *
 */
long GetListKeyByRank(LinkedListPtr list, long rank)
{
	short i;
	ListEntryPtr cur;
	
	if (!list) return 0;
	if ((rank < 1) || (rank > list->numEntries)) return 0;
	
	i=1; cur = list->lList; // i=1 not 0 *MDT 2/14/96
	while ((i != rank) && (cur != 0)) {
		cur = cur->next;
		i++;
	}
	
	return cur->key;
}

/* LinkedListCount
 * Returns the number of items in the list
 */
long LinkedListCount(LinkedListPtr list) 
{
	long ret =0;
	
	if (list) {
		ret = list->numEntries;
	}
	return ret;
}

/* LinkedListIsEmpty
 * Returns the boolean interperetation of LinkedListCount()
 */
Boolean LinkedListIsEmpty(LinkedListPtr list)
{
	return (!LinkedListCount(list));
}
