/*
progress_dlog.c

This is my implementation of a program found on the Internet named "Finder Progress"
or some such. I made some modifications to it, and created a pascal "unit" for use
with Chris Knowles' and my original Fractal programs.

With the change to C, rather than going back to the original code, I have decided
to port my version over - as I believe my changes were valuable. As such, the 
original pascal comments remain here.

M Trent 3/31/94

CHANGE: 4/95 IT NOW REQUIRES GWORLDS!!

	Function StartProgress (prompt: Str255): Boolean;
{ StartProgress gets the progress Dialog window, Draws it, and sets it to the active	}
{ port. Thus, it's critically important that after calling this routine, the host			}
{ program does no drawing, changing ports, etc after calling StartProgress, until		}
{ EndProgress is called. Promt is a string drawn just above the progress bar. If the	}
{ Dialog can't be found or drawn, the procedure will return false before changing		}
{ the active port.																				}

{ Note that the progress Dialog must be present in the programs Resource Fork as a	}
{ Dialog, ID 369. Progress bar rectangle must be a user item, with an item ID of 1; 	}
{ the rectangle can have any dimensions. The second item must be a StaticText item,	}
{ ID number 2; it should contain the string "^0".											}

	Procedure UpdateProgress (percentage: integer);
{ Update progress should be called often enough to update the length of the progress	}
{ bar in the Dialog. Note this isn't flicker-free animation :( 								}

	Procedure EndProgress;
{ UpdateProgress destroys the progress Dialog window, and restores whatever port was	}
{ active when StartProgress was called. Call this when the host program is done with		}
{ the progress Dialog.																				}

*/

#include <Dialogs.h>
#include <QDOffscreen.h>
#include <ToolUtils.h>
#include "Progress.h"

#define kGrayProgressColorRed  17476
#define kGrayProgressColorGreen 17476
#define kGrayProgressColorBlue 17476

#define kPurpleProgressColorRed 52428
#define kPurpleProgressColorGreen 52428
#define kPurpleProgressColorBlue 65535

//#define kModalProgressDialogID 369 pass it in now.
#define kDialogProgressItem 1

#define kColorBitDepth 8

/* global-in-unit variables */
Boolean		gColor;
DialogPtr	gD;
GDHandle 	gSaveDevice;
Rect		gR;
short		gDepth;

/* private routines */
Boolean ColorQDIsPresent (void);
short BitDepth(void);
Rect DItemRect (DialogPtr dialog, short item);


/*	ColorQDIsPresent											*/
/*	returns true if color quickdraw is present, false otherwise	*/
/* 	This really is too old - I should use the Gestalt Manager	*/

Boolean ColorQDIsPresent (void)
{
	SysEnvRec theWorld;
	OSErr err;

	err = SysEnvirons(1, &theWorld);
	return (theWorld.hasColorQD);
}


/*	BitDepth 													*/
/*	returns the bitdepth of the main monitor					*/

short BitDepth(void)
{
	short 			bits = 1;
	GDHandle 		mainDevice;
	PixMapHandle	pixMap;

	if (gColor) {
		mainDevice = GetMainDevice();
		pixMap = (*mainDevice)->gdPMap;
		bits = (*pixMap)->pixelSize;
	}
	
	return (bits);
}

Rect DItemRect (DialogPtr dialog, short item)
{
	short 	itemType;
	Handle	itemHandle;
	Rect 	itemRect;

	GetDItem(dialog, item, &itemType, &itemHandle, &itemRect);
	return (itemRect);
}


Boolean StartProgress (short dID, Str255 prompt)
{
	CursHandle crsr;
	CGrafPtr	savePort;
	GDHandle saveDevice;
	short iType;
	Handle iHandle;
	Rect iRect;
	
	GetGWorld(&savePort,&saveDevice);
	gSaveDevice = saveDevice;
	
	gColor = ColorQDIsPresent();
	gDepth = BitDepth();
	gD = GetNewDialog(dID, 0L, (WindowPtr)-1L);
	if (gD) {
		SetGWorld((CGrafPtr)gD,gSaveDevice);
		GetDItem(gD, 2, &iType, &iHandle, &iRect);
		SetIText(iHandle, prompt);
		//ParamText(prompt, "\p", "\p", "\p");

		ShowWindow(gD);
		DrawDialog(gD);

		/*crsr = GetCursor(watchCursor);
		HLock((Handle)crsr);
		SetCursor((*crsr));
		HUnlock((Handle)crsr);*/ // 4/24/96
		
		gR = DItemRect(gD, kDialogProgressItem);
		SetGWorld(savePort, saveDevice);
		return true;
	} else {
		return false;
	}
}

void UpdateProgress (short percentage)
{
	RGBColor rgb, forColor;
	Rect theRect, fillRect, emptyRect;
	PenState savePen;
	short offset;
	CGrafPtr	savePort;
	GDHandle saveDevice;
	
	GetGWorld(&savePort,&saveDevice);

	SetGWorld((CGrafPtr)gD,gSaveDevice);

	theRect = gR;
	fillRect = gR;
	emptyRect = gR;

//	{ save the pen state and color }
	GetPenState(&savePen);
	if (gColor) 
		GetForeColor(&forColor);
	PenNormal();
	//rgb.red = rgb.green = rgb.blue = 0x0000;
	//RGBBackColor(&rgb);

//	{ first frame the rectangle }
	if (gColor) {
		rgb.red = 0;
		rgb.green = 0;
		rgb.blue = 0;
		RGBForeColor(&rgb);
	} else
		ForeColor(blackColor);

	FrameRect(&theRect);

//	{ figure out the percentage }
	InsetRect(&theRect, 1, 1);
	InsetRect(&fillRect, 1, 1);
	InsetRect(&emptyRect, 1, 1);
	offset = ((theRect.right - theRect.left) * percentage) / 100;

//	{ first draw the filled in portion }
	if (percentage != 0) { // MDT 3/17/96
		fillRect.right = fillRect.left + offset;
		if (gColor) {
				rgb.red = kGrayProgressColorRed;
				rgb.green = kGrayProgressColorGreen;
				rgb.blue = kGrayProgressColorBlue;
				RGBForeColor(&rgb);
		} else
			ForeColor(blackColor);

		PaintRect(&fillRect);
	}// MDT 3/17/96

//	{ now draw the unfilled portion }
	if (percentage != 0) emptyRect.left = fillRect.right + 1; // MDT 3/17/96
	if (gColor) {
			rgb.red = kPurpleProgressColorRed;
			rgb.green = kPurpleProgressColorGreen;
			rgb.blue = kPurpleProgressColorBlue;
			RGBForeColor(&rgb);
	} else
		ForeColor(whiteColor);

	PaintRect(&emptyRect);

//{ restore pen settings, colors, restore to black when not in color }
	SetPenState(&savePen);
	if (gColor) 
		RGBForeColor(&forColor);
	else
		ForeColor(blackColor);
		
	SetGWorld(savePort, saveDevice);
}

void EndProgress(void)
{
//{ clean things up }
	//SetPort(gD);
	//InitCursor();
	//SetCursor(&qd.arrow); 4/24/96
	ParamText("\p", "\p", "\p", "\p");
	DisposDialog(gD);
	//SetPort(gSavePort);
}

void ChangeCaption(Str255 caption)
{
	short iType;
	Handle iHandle;
	Rect iRect;
	CGrafPtr	savePort;
	GDHandle saveDevice;
	
	if (!gD) return;
	
	GetGWorld(&savePort,&saveDevice);
	SetGWorld((CGrafPtr)gD,gSaveDevice);
	
	GetDItem(gD, 2, &iType, &iHandle, &iRect);
	EraseRect(&iRect);
	SetIText(iHandle, caption);

	SetGWorld(savePort, saveDevice);
}