/* 
 * Scen List.c
 *
 * Routines for managing the Scenario List.
 * This was originally implemented in Pebbles as a class - I'm not 
 * convinced it will be as useful as a C library...
 *
 * These routines are for general list manipulation - i.e., adding and
 * removing items from the list, selecting items, etc.
 */

#include "Scen LDEF.h"
#include "Scen List.h"
#include "Scenario.h"

#include "Map Window.h"
#include "KeyAscii.h"

#define kLDEFResID		0

// useful defines for working with lists.
#define		kDefaultList	0
#define		kDrawList		true
#define		kDontDrawList	false
#define		kNoGrowBox		false
#define		kNoHScroll		false
#define		kHScroll		true
#define		kVScroll		true

/* Globals */
ListHandle gScenList = nil;
//short gLastCellIndex = 0;
//Boolean gListDirty = true;

// "Virutal" list globals and constants.
#define		kOutdoorMapList	0
#define 	kDungeonMapList 1
#define 	kMonsterList	2
#define 	kItemList		3		

// code 

OSErr NewScenList(WindowPtr w,Rect *r)
{
	Rect	dataBounds, rView = *r;
	Point	cSize;
	short i;
	
	SetRect(&dataBounds, 0,0,1,0);
	rView.right -= 15; // make room for scroll bars 
	
	SetPt(&cSize, (rView.right-rView.left),16 );
		
	gScenList = LNew(&rView, &dataBounds, cSize, kLDEFResID, w, kDontDrawList, 
					   kNoGrowBox, kHScroll, kVScroll);
	if (!gScenList) {
		return -1; // do some error thing here .. 
	}
	(*gScenList)->selFlags = lOnlyOne;
	(*gScenList)->refCon = (long) ScenList;		// THIS IS DAMN IMPORTANT!!!
	//gLastCellIndex = 0;
	
	BuildList();
	
	return noErr;
}

void DisposeScenList(void)
{
	LDispose(gScenList);
}

void BuildList(void)
{
	Cell	cell, theCell={0,0};
	short lastCellIndex;	

	LDoDraw(false, gScenList);

	for (lastCellIndex = 0; lastCellIndex < kNumberOfOutsideMaps; lastCellIndex++) {
		(void) LAddRow(1,lastCellIndex+1,gScenList);
		SetPt(&cell, 0, lastCellIndex);
		LSetCell(&(gScenario.outsideMaps[lastCellIndex].mapName[1]),
			gScenario.outsideMaps[lastCellIndex].mapName[0],
			cell,
			gScenList); 
	}
	
	LSetSelect(true, theCell, gScenList); // select first element
	LDoDraw(true, gScenList);
}

void ResizeScenList (Rect *r) {
	short width,height;
	
	width = r->right - r->left - 15;
	height = r->bottom - r->top - 15; // should be 30 eventually
	
	LSize(width, height, gScenList);
}

void	UpdateScenList(WindowPtr w)
{
	RgnHandle rgn = w->visRgn;
	LUpdate(rgn,gScenList);
}

void ActivateScenList(WindowPtr w, Boolean activate)
{
	LActivate(activate,gScenList);
}

// Pre: window has been set w/ SetPort();
void HandleScenListClick(EventRecord *e)
{
	Point	localPt = e->where;
	short modifiers = e->modifiers;
	Cell	theCell={0,0};
	WindowPtr w;

	GlobalToLocal(&localPt);	
	if(LClick(localPt, modifiers, gScenList)) {
		if(LGetSelect(true, &theCell, gScenList)) { 
			w = NewMapWindow(theCell.v);
			/*if (gMapWindow == nil)
				(void) NewMapWindow();
			SetMapID(theCell.v);*/
			CalcAndDrawMapWindow(w);
		}
	}
}

void HandleScenListKey(EventRecord *e, short theChar)
{
	Cell	theCell,cell;
	WindowPtr w;
	
	SetPt((Point*)&theCell,0,0);

	if (LGetSelect(true, &theCell, gScenList)) {
		switch (theChar) {
		case kReturnKey:
		case kEnterKey:
			w = NewMapWindow(theCell.v);
/*			if (gMapWindow == nil)
				(void) NewMapWindow();
			SetMapID(theCell.v);*/
			CalcAndDrawMapWindow(w);
			break;
		
		case kUpKey:
		case kLeftKey:
			cell = theCell; cell.v -= 1;
			if (!(cell.v < 0)) {
				LSetSelect(false, theCell, gScenList);
				LSetSelect(true, cell, gScenList);
			}
			break;
			
		case kDownKey:
		case kRightKey:
			cell = theCell; cell.v += 1;
			if (cell.v < kNumberOfOutsideMaps) {
				LSetSelect(false, theCell, gScenList);
				LSetSelect(true, cell, gScenList);
			}
			break;
		
		default:
			;
		}
	}
}
