/* 
 * Stack.c - normal linked-list stack
 */
/* Hybrid modification. 12/26/95
 */
#include "Stack.h"

void NewStack(StackPtr *stack)
{
	StackPtr cur;
	
	cur = (StackPtr)NewPtr(sizeof(StackMaster));
	
	cur->stack = nil;
	cur->numEntries = 0;
	
	*stack = cur;
}

void DisposeStack(StackPtr stack)
{
	StackEntryPtr cur;
	
	if (stack == nil) return;
	
	while (cur = stack->stack) {
		stack->stack = stack->stack->next;
		DisposePtr((Ptr)cur);
	}
	stack->numEntries = 0;

	DisposePtr((Ptr)stack);
}

void StackPush(StackPtr stack, long data)
{
	StackEntryPtr cur;
	short index;
	
	if (stack == nil) return;

	index = stack->numEntries % kStackArraySize;
	// push a new item on stack if needed.
	if (index == 0) {
		cur = (StackEntryPtr)NewPtr(sizeof(StackEntry));	
		cur->next = stack->stack;
		stack->stack = cur;
	}
	stack->stack->data[index] = data;
	stack->numEntries++;
}

long StackPop(StackPtr stack)
{
	long ret;
	StackEntryPtr cur;
	short index;
	
	if (stack == nil) return nil;
	if (stack->stack == nil) return nil;
	if (stack->numEntries == 0) return nil;
	
	index = (stack->numEntries % kStackArraySize)-1; // minus one intentional
	if (index == -1) index = (kStackArraySize -1 ); // fudge factor
	
	ret = stack->stack->data[index];
	stack->numEntries--;

	//pop the stack if necessary
	if (index == 0) {
		cur = stack->stack;
		stack->stack = stack->stack->next;
		DisposePtr((Ptr)cur);
	}

	return ret;
}

long StackPeek(StackPtr stack)
{
	short index;
	
	if (stack == nil) return nil;
	if (stack->stack == nil) return nil;
	
	index = stack->numEntries % kStackArraySize-1; // minus one intentional
	if (index == -1) index = (kStackArraySize -1 ); // fudge factor
	
	return stack->stack->data[index];
}

long StackCount(StackPtr stack)
{
	if (stack == nil) return nil;
	if (stack->stack == nil) return nil;
	
	return stack->numEntries;
}

Boolean StackIsEmpty(StackPtr stack) // 'cause angela said so.
{
	if (stack == nil) return true;
	return (stack->numEntries == 0);
}