/*
 * Tiles.c
 *
 * Routines for managing and manipulating Ultima's tiled graphics.
 */

#include <QDOffscreen.h>
 
#include "MMR.h"
#include "Tiles.h"
#include "Scenario.h"

#define		kTilesPictID	0

//GWorldPtr gTilesWorld = nil;

/* LoadTiles
 * Load the sprites from the current resource file.
 */
/*OSErr LoadTiles(void)
{
	PicHandle tilePict = nil;
	OSErr err;
	Rect r;
	
	tilePict = GetPicture(0);
	if (!tilePict) {
		err = 1;
		goto bail;
	}
	
	r = (*tilePict)->picFrame;
	OffsetRect(&r,-r.left, -r.top); // zero left and top edges of rect.
	
	err = MakeGWorld(&gTilesWorld, &r, 8);
	if (err) goto bail;
	
	DrawMyPict(gTilesWorld,tilePict);
	
	ReleaseResource((Handle)tilePict); // clean up after ourselves.
	return noErr;
bail:
	if (tilePict) ReleaseResource((Handle)tilePict);
	DisposeTiles();
	return err;
}*/

/* DisposeTiles
 * This frees the offscreen world containing our sprites.
 */
/*void DisposeTiles(void)
{
	if (gTilesWorld) DisposeGWorld(gTilesWorld);
	gTilesWorld = nil;
}*/

/* PlotTile
 * Plots a given tile into a specific rect on the destination World.
 */
/*void PlotTile(short tileID, GWorldPtr destWorld, Rect *destRect)
{
	Rect sourceRect;
	
	tileID = (short)(tileID/4); // sloppy.

	SetRect(&sourceRect,0, 0+(32 * tileID), 32, 32+(32 * tileID));

	CopyBits(&((GrafPtr)gTilesWorld)->portBits, &((GrafPtr)destWorld)->portBits,
			 &sourceRect, destRect,  srcCopy, 0);
}*/


/*
 * GLOBAL FREE CODE
 */


/* LoadTiles
 * Load the sprites from the current resource file.
 */
OSErr LoadTiles(GWorldPtr *tilesWorld)
{
	PicHandle tilePict = nil;
	OSErr err;
	Rect r;
	
	tilePict = GetPicture(kTilesPictID);
	if (!tilePict) {
		err = 1;
		goto bail;
	}
	
	r = (*tilePict)->picFrame;
	OffsetRect(&r,-r.left, -r.top); // zero left and top edges of rect.
	
	err = MakeGWorld(tilesWorld, &r, 8);
	if (err) goto bail;
		
	DrawMyPict(*tilesWorld,tilePict);
	
	ReleaseResource((Handle)tilePict); // clean up after ourselves.
	return noErr;
bail:
	if (tilePict) ReleaseResource((Handle)tilePict);
	DisposeTiles(tilesWorld);
	return err;
}

/* DisposeTiles
 * This frees the offscreen world containing our sprites.
 */
void DisposeTiles(GWorldPtr *tilesWorld)
{
	if (*tilesWorld) DisposeGWorld(*tilesWorld);
	*tilesWorld = nil;
}


/* PlotTile
 * Plots tile of ID tileID into world destWorld.  destRect is the area to blit into.
 * If destWorld is nil, PlotTile will plot to the current port, whatever that is.
 * If destWorld points to a valid GWorld, it should already be locked down!!
 *
 * Keep in mind this routine needs to be as efficient as possible.
 */
void PlotTile(GWorldPtr tiles, short tileID, GWorldPtr destWorld, Rect *destRect)
{
	Rect sourceRect;
	GrafPtr grafPtr;
	if (destWorld) {
		grafPtr = (GrafPtr)destWorld;
	} else {
		GetPort(&grafPtr);
	}
	
	tileID = (short)(tileID/4); // sloppy.

	//SetRect(&sourceRect,0, 0+(32 * tileID), 32, 32+(32 * tileID));
	sourceRect.left = 0;
	sourceRect.right = 32;
	sourceRect.top = 0+(32 * tileID);
	sourceRect.bottom = 32+(32 * tileID);

	if (tiles) {
		//SetStdForeNBackColor();
		CopyBits(&((GrafPtr)tiles)->portBits, &grafPtr->portBits,
				 &sourceRect, destRect,  srcCopy, 0);
	} else {
		PaintRect(destRect);
	}
}