/*
 * ToolPaletteCDEF
 *
 * Control DEF for a standard Tool Palette. The details:
 *
 * ControlRecord:
 *		contrlRect	-	left and top are set correctly; right and bottom are set by
 *						the control when created.
 *		contrlMin	-	set to -1 
 *		contrlMax 	-	Number of buttons. Set only when creating the control;
 *							- do not change!
 *		contrlValue	-	Selected button. Range: -1 -- contrlMax. -1 means create a 
 *						floating window, 0 means no control was selected, 1 - contrlMax
 *						specifies active control.
 *		refCon		-	pointer to ControlDataRec. During initialization, this pointer is
 *                      moved to contrlData.
 *
 *	ControlDataRec is not set automatically! It contains:
 *		tools		-	GWorldPtr containing graphics. The graphic format is simple:
 *						Items are 24x21 pixels in size; items will be drawn in two
 *                      columns: up position (left) and down position (right). The 
 *                      control will be drawn in two columns also, simply alternating
 *                      left-right when positioning the controls. i.e.:
 *                      	1 2
 * 							3 4 etc ... 
 *		numTools	-	Number of tools in GWorld. This is set from contrlMax when the 
 *						control is created. It is never updated after creation.
 *		enabled		-	Array of enabled controls. Enabled controls will be drawn using
 *						graphics defined in the GWorld. Disabled controls will be drawn
 *						as a light grey (0xEEEE) rectangle w/ a black frame. Disabled
 *						controls will not track.
 *		windowPt	-	The point in which to draw a floating window.  See contrlValue.
 */

#include <QDOffscreen.h>

#include "ToolPaletteCDEF.h"

pascal long main(short varCode,ControlHandle theControl,short message,long param);
void DrawToolPalette(ControlHandle theControl, ControlDataRec *data);
void DrawTool(short tool, ControlHandle theControl, ControlDataRec *data);
short ToolPaletteItemHit(Rect *rect, Point hitPt,ControlDataRec *data);
void ToolPaletteAction(short varCode, ControlHandle theControl, short tool, ControlDataRec *data);
void SetGrey(RGBColor *rgb, short shade);

pascal long main(short varCode,ControlHandle theControl,short message,long param)
{
	long		result = 0;
	//SignedByte	saveHState = HGetState((Handle)theControl);
	Point		testPt;
	Rect		r;
	ControlDataRec	*data;
	short shortParam;
	RGBColor fore,back,grey;
	short controlWidth, controlHeight;
	
	// lock stuff down, and other initializations
	HLock((Handle)theControl);
	SetGrey(&fore,0x0000);
	SetGrey(&back,0xFFFF);	
	
	switch (message) {
	case initCntl:
		// set the simple stuff.
		data = (ControlDataRec*)(*theControl)->contrlRfCon;  
		(*theControl)->contrlMin = -1;

		// set the control rect.
		controlWidth = (2 * data->borderWidth) + ((data->columns * data->itemWidth - (data->columns-1)));
		controlHeight= (2 * data->borderHeight) + ((data->rows * data->itemHeight - (data->rows-1)));
		r = (*theControl)->contrlRect;
		r.right = r.left + controlWidth;
		r.bottom = r.top + controlHeight;
		(*theControl)->contrlRect = r;
		
		// set contrlAction
		(*theControl)->contrlAction = (ControlActionUPP)NewControlActionProc(-1L);
		
		//move contrlRefCon to contrlData
		(*theControl)->contrlData = (Handle)(*theControl)->contrlRfCon;
		(*theControl)->contrlRfCon = 0L;
		break;
	case drawCntl:
		// set data
		data = (ControlDataRec*)(*theControl)->contrlData;
		r = (*theControl)->contrlRect;
		InsetRect(&r,data->borderWidth,data->borderHeight);
		FrameRect(&r);
		
		if ((*theControl)->contrlVis == 255) {
			if ((*theControl)->contrlHilite != 255) {
				DrawToolPalette(theControl,data);
			} else {
				InsetRect(&r,1,1);
				SetGrey(&grey,0xEEEE);
				RGBForeColor(&grey);
				PaintRect(&r);
			}
		}
		break;
	case testCntl:
		if ((*theControl)->contrlHilite != 255) {
			data = (ControlDataRec*)(*theControl)->contrlData;
			testPt.h = param & 0xFFFF;
			testPt.v = (param >> 16) & 0xFFFF;
			
			r = (*theControl)->contrlRect;
			result = ToolPaletteItemHit(&r, testPt,data) ;
			// here's where we can do outline dragging...
		}
		break;
	case calcCRgns:			// 24 bit pre-processing
		param &= 0x00FFFFFF;
		// FALL THROUGH!!
	case calcCntlRgn:		// 32 bit call
		r = (*theControl)->contrlRect;
		RectRgn((RgnHandle)param,&r);
		break;
	case dispCntl:
		DisposeRoutineDescriptor((ControlActionUPP)(*theControl)->contrlAction);
		break;
	case autoTrack:
		data = (ControlDataRec*)(*theControl)->contrlData;
		shortParam = (short)param;
		ToolPaletteAction(varCode, theControl, shortParam, data);
		break;
	default:
		;
	}

	RGBForeColor(&fore);
	RGBBackColor(&back);
	//HSetState((Handle)theControl,saveHState);
	HUnlock((Handle)theControl);
	return result;
}

// DrawToolPalette - draws entire palette.
void DrawToolPalette(ControlHandle theControl, ControlDataRec *data)
{
	short i;
	
	if (!data) return;
	if (!theControl) return;
	
	(void)LockPixels(data->tools->portPixMap);
	for (i = 0; i < data->numTools; i++) {
		DrawTool(i + 1, theControl, data);
	}
	UnlockPixels(data->tools->portPixMap);
}

// assumes portPixMap locked down; assumes fore and back colors are cached.
// range tool = 1 - numTools. if tool = 0, it won't draw.
void DrawTool(short tool, ControlHandle theControl, ControlDataRec *data)
{
	Rect src,dest,r;
	GrafPtr	grafPtr;
	RGBColor	grey, black, white;
	Boolean *enabled;

	if ((tool < 1) || (tool > data->numTools)) return;
	if (!data) return;
	if (!theControl) return;

	SetGrey(&grey,0xDDDD);
	SetGrey(&black,0x0000);
	SetGrey(&white,0xFFFF);
	
	r = (*theControl)->contrlRect;
	enabled = data->enabled;
		
	// set dest (current window) rectangle - window origin set w/ contrlRect.
	dest.left = ((tool-1) % data->columns) * ((data->itemWidth)-1); // -1 so the edges overlap.
	dest.top  = ((tool-1) / data->columns) * ((data->itemHeight)-1);
	dest.right = dest.left + (data->itemWidth);
	dest.bottom = dest.top + (data->itemHeight);
	OffsetRect(&dest, r.left + data->borderWidth, r.top + data->borderHeight);

	if (((*theControl)->contrlHilite != 255) && enabled[tool-1]) { // if enabled (tool starts at 1, C array starts at 0)
		// set src (GWorld) rectangle - zero origin
		SetRect(&src, 0, (tool-1) * data->itemHeight, data->itemWidth, (tool-1) * data->itemHeight + data->itemHeight);
		if (tool == ((*theControl)->contrlValue) ) {	// if selected
			OffsetRect(&src,data->itemWidth,0);				// choose selected shape
		}
		
		grafPtr = (*theControl)->contrlOwner;

		RGBForeColor(&black); RGBBackColor(&white);
		CopyBits(&((GrafPtr)(data->tools))->portBits, &grafPtr->portBits, 
				&src, &dest, srcCopy, 0);
	} else { // if not enabled
		RGBForeColor(&black);
		FrameRect(&dest);
		InsetRect(&dest,1,1);
		RGBForeColor(&grey);
		PaintRect(&dest);
	}
}


// returns tool number hitPt is in, or returns 0 for no tool. rect is local to the 
// active GrafPort.
short ToolPaletteItemHit(Rect *rect, Point hitPt,ControlDataRec *data)
{
	short item = 0;
	Rect r;
	
	hitPt.v -= 1; hitPt.h -=1 ;//kluge!!
	
    // insure our hit pt is in the control - not including the border!
	r = *rect;
	InsetRect(&r,data->borderWidth,data->borderHeight);
	r.bottom -= 1; r.right -= 2; // kluge! the last grid lines!
	
	if (PtInRect( hitPt, &r)) {
		// make the control's left/top our co-ordinate origin:
		hitPt.h -= r.left; hitPt.v -= r.top;	// zero out hitPt
		OffsetRect(&r, -r.left, -r.top);		// zero out r

		// item must equal AT LEAST 1.
		item = (hitPt.v / (data->itemHeight-1) * data->columns) + (hitPt.h / (data->itemWidth-1)) + 1; // -1 because they overlap
	}	

	return item;
}

// this guy sets our control value.... 
void ToolPaletteAction(short varCode, ControlHandle theControl, short tool, ControlDataRec *data)
{
	short val;
	GrafPtr	grafPtr;
	//RGBColor fore,back;
	Point mouse,pt;
	Rect r;
	
	if (!data) return;
	if (!theControl) return;

	// get the damn mouse value ourselves!!
	GetMouse(&mouse);
	tool = ToolPaletteItemHit(&(*theControl)->contrlRect, mouse,data); 
	
	val = (*theControl)->contrlValue; // val can be zero!
	
	if ((tool == 0) && (varCode == 1)) {
		r = (*theControl)->contrlRect;
		pt.h = r.left; pt.v = r.top;
		LocalToGlobal(&pt);
		OffsetRect(&r,pt.h-r.left, pt.v-r.top);
		// do the outline dragging
		// return -1
	}
	
	if (val != tool) {
		(*theControl)->contrlValue = tool;
	
		(void)LockPixels(data->tools->portPixMap);
		DrawTool(val, theControl, data);
		DrawTool(tool, theControl, data);
		UnlockPixels(data->tools->portPixMap);
	}
}

void SetGrey(RGBColor *rgb, short shade)
{
	rgb->red = rgb->green = rgb->blue = shade;
}
