/*
 * Tools.c
 */

#include "MMR.h"
#include "Map Window.h"
#include "Scenario.h"
#include "Tools.h"
#include "Tiles.h"
#include "TileMenuCommon.h"
#include "Stack.h"
#include "WatchCursor.h"
#include "Monsters.h"


Boolean DoEraser(Point initialPt, Point mapPt, MapWindowData *mwg)
{
	Rect r;
	long h,v;
	unsigned char *a,*b; 
	long mpSize = gScenario.outsideMapsSize[mwg->mapID].h; // mdt 3/11/96
	
	a = (unsigned char *)*gScenario.outsideMaps[mwg->mapID].map;
	b = (unsigned char *)*gScenario.outsideMaps[mwg->mapID].mapCopy;

	// set map array coordinates
	h = mapPt.h;
	v = mapPt.v;
	
	// calculate the area rect.
	r.top = (v * 32) - GetCtlValue(mwg->mapVScroll);
	r.left = (h * 32) - GetCtlValue(mwg->mapHScroll) + (kToolSmallOffset * 2) + kToolWidth;
	r.bottom = r.top + 32;
	r.right = r.left + 32;

	// Set the map array
	a[(mpSize*v) + h] = b[(mpSize*v) + h];

	// draw new tile.
	PlotTile(gScenario.tiles, b[(mpSize*v) + h],(GWorldPtr)(mwg->mapWindow),&r);
	
	return true; // so we remember to recalculate.
}

Boolean DoPencil(Point initialPt, Point mapPt, MapWindowData *mwg)
{
	Rect r;
	long h,v;
	unsigned char *a; 
	long mpSize = gScenario.outsideMapsSize[mwg->mapID].h; // mdt 3/11/96
	
	a = (unsigned char *)*gScenario.outsideMaps[mwg->mapID].map;

	// set map array coordinates
	h = mapPt.h;
	v = mapPt.v;
	
	// calculate the area rect.
	r.top = (v * 32) - GetCtlValue(mwg->mapVScroll);
	r.left = (h * 32) - GetCtlValue(mwg->mapHScroll) + (kToolSmallOffset * 2) + kToolWidth;
	r.bottom = r.top + 32;
	r.right = r.left + 32;

	// Set the map array
	a[(mpSize*v) + h] = (unsigned char)(mwg->tileType);

	// draw new tile.
	PlotTile(gScenario.tiles,(unsigned char)(mwg->tileType),(GWorldPtr)(mwg->mapWindow),&r);
	
	return true; // so we remember to recalculate.
}


Boolean DoEyeDropper(Point initialPt, Point mapPt, MapWindowData *mwg)
{
	Rect r;
	long h,v,item;
	unsigned char *a; 
	long mpSize = gScenario.outsideMapsSize[mwg->mapID].h; // mdt 3/11/96
	
	a = (unsigned char *)*gScenario.outsideMaps[mwg->mapID].map;

	// set map array coordinates
	h = mapPt.h;
	v = mapPt.v;
	
	// Set the Tile Type
	item = (short)a[(mpSize*v) + h];
	SetTileType(mwg->mapWindow,item);
	item = item / 4 + 1;
	CheckTMItem(item);
	DrawTilePopup(mwg->mapWindow);

	return false; // we Don't recaculate.
}

// initialPt is a (short) containing typeToReplace
Boolean DoPaintBucket(Point initialPt, Point mapPt, MapWindowData *mwg)
{
	Rect r;
	long h,v;
	long fillOrder,i,typeToReplace;
	unsigned char *a;
	Boolean fillSelect;
	Point savePt;
	long	tmp,ticks,origTicks;
	StackPtr stack;
	long mpSize = gScenario.outsideMapsSize[mwg->mapID].h; // mdt 3/11/96
	
	origTicks = ticks = TickCount() + 20;
	
	a = (unsigned char *)*gScenario.outsideMaps[mwg->mapID].map;

	NewStack(&stack);
	StackPush(stack,(*(long*)&mapPt));
	
	while (!StackIsEmpty(stack)) {
		if (TickCount() > ticks) {
			DoWatchCursor(mwg->mapWindow,kWatchSpin);
			ticks = TickCount() + 10;
		}		
	
		tmp = StackPop(stack);
		mapPt = *(Point *)&tmp;
		
		// set map array coordinates
		savePt = mapPt;
		h = mapPt.h;
		v = mapPt.v;
		typeToReplace = initialPt.h;
		fillSelect = initialPt.v;
		
		// bail if co-ordinate is not the tile-type we are replacing
		if (typeToReplace != (short)a[(mpSize*v) + h])
			continue;
			
		// also bail if the current tile is already the proper type (optomization)
		if (mwg->tileType == (short)a[(mpSize*v) + h])
			continue; 
			
		// also bail if we're about to draw on the wrong side of the fill selection.
		// If we start to fill inside the fill rect, we want the fill confined to the fill rect.
		// but if we start the fill outside the select rect, we want the fill confined to the
		// area outside the select rect. 
		// Note if there is no select rect (selectPts.left = -1) fillSelect and PtInRect() will 
		// both equal false.
		if (fillSelect != PtInSelectedRect(mwg->mapWindow,mapPt))
			continue;
	
		// draw ourselves: Set the map array
		a[(mpSize*v) + h] = (unsigned char)(mwg->tileType);

		// Push pts onto the stack.		
		mapPt.h -= 1;
		if ((mapPt.h>-1) && (typeToReplace == (short)a[(mpSize*mapPt.v) + mapPt.h]))
				StackPush(stack,(*(long*)&mapPt));
		
		mapPt = savePt; mapPt.v -= 1;
		if ((mapPt.v>-1) && (typeToReplace == (short)a[(mpSize*mapPt.v) + mapPt.h]))
				StackPush(stack,(*(long*)&mapPt));
		
		mapPt = savePt; mapPt.h += 1;
		if ((mapPt.h<mpSize) && (typeToReplace == (short)a[(mpSize*mapPt.v) + mapPt.h]))
				StackPush(stack,(*(long*)&mapPt));
		
		mapPt = savePt; mapPt.v += 1;
		if ((mapPt.v<mpSize) && (typeToReplace == (short)a[(mpSize*mapPt.v) + mapPt.h]))
				StackPush(stack,(*(long*)&mapPt));
	}
	
	DisposeStack(stack);
	
	if (origTicks != ticks ) DoWatchCursor(mwg->mapWindow,kWatchOff);
	return true; // so we remember to recalculate.
}

//our cursor is now abstracted from selectedPts.
Boolean DoLetter(Point initialPt, Point mapPt, MapWindowData *mwg)
{
	if (mwg->selectedPts.left != -1) {
		Rect r;
		MPtoQDRect(mwg->mapWindow,&mwg->selectedPts,&r);
		InvertSelectedRect(mwg->mapWindow);
	}

	SetRect(&mwg->selectedPts,mapPt.h,mapPt.v,mapPt.h,mapPt.v);
	InvertSelectedRect(mwg->mapWindow);
	
	return false; 
}

// letter range -1 to 20. 0-20 = valid letters, -1 = draw nothing, -2 erase mode.
void DoLetterKeyPress(short letter, Boolean adjust, MapWindowData *mwg)
{
	short	letterID;
	Rect 	r,mapRect;
	unsigned char *a,*b; 
	RgnHandle save, clip;
	Boolean dirty = false;
	long mpSize = gScenario.outsideMapsSize[mwg->mapID].h; // mdt 3/11/96
	
	//if (letter == -1) return; // bogus letter
	//if (mwg->cursor.h == -1) return; // bogus cursor
	if (mwg->selectedPts.left == -1) return; // bogus cursor
	
	if (letter == 10) return; // there is no letter j
	if (letter == 11) return; // there is no letter k
	if (letter == 17) return; // there is no letter q
	if (letter == 21) letter = 10; // u thinks it's j
	if (letter == 25) letter = 11; // y thinks it's k
	if (letter > 20) return; // it's just not there!!
	
	HLock(gScenario.outsideMaps[mwg->mapID].map);
	//HLock(gScenario.outsideMaps[mwg->mapID].mapCopy);
	a = (unsigned char *)*gScenario.outsideMaps[mwg->mapID].map;
	//b = (unsigned char *)*gScenario.outsideMaps[mwg->mapID].mapCopy;
	
	save = NewRgn();
	clip = NewRgn();
	GetClip(save);
	CalcMapRect(mwg->mapWindow,&mapRect);
	RectRgn(clip, &mapRect);
	SetClip(clip);

	/*if (letter == -2) { // erase mode
		 letterID = (short)b[(64*mwg->cursor.v) + mwg->cursor.h];
	} else if (letter == -1) {
		letterID = -1;
	} else {*/
	letterID = (letter + 37) * 4;
	/*}*/
		
	// calculate the area rect.
	/*r.top = (mwg->cursor.v * 32) - GetCtlValue(mwg->mapVScroll);
	r.left = (mwg->cursor.h * 32) - GetCtlValue(mwg->mapHScroll) + (kToolSmallOffset * 2) + kToolWidth;
	r.bottom = r.top + 32;
	r.right = r.left + 32;*/
	MPtoQDRect(mwg->mapWindow,&mwg->selectedPts,&r);

	//if (letterID != -1) {
		//   Set the map array
		a[(mpSize*mwg->selectedPts.bottom) + mwg->selectedPts.left] = (unsigned char)letterID;
	
		// draw new tile.
		SetStdForeNBackColor();
		PlotTile(gScenario.tiles,(unsigned char)letterID,(GWorldPtr)(mwg->mapWindow),&r);
	//}
	
	// adjust our scrollbars and redraw window if cursor is off screen.
	if (r.left < mapRect.left) {
		SetCtlValue(mwg->mapHScroll, GetCtlValue(mwg->mapHScroll) + (r.left - mapRect.left));
		dirty = true;
	} else if (r.right > mapRect.right) {
		//SetCtlValue(gMWGlobals.mapHScroll, GetCtlValue(gMWGlobals.mapHScroll) + (r.right - mapRect.right));
		SetCtlValue(mwg->mapHScroll, GetCtlValue(mwg->mapHScroll) + (r.left - mapRect.left - 32));
		dirty = true;
	}	
	if (r.top < mapRect.top) {
		SetCtlValue(mwg->mapVScroll, GetCtlValue(mwg->mapVScroll) + (r.top - mapRect.top));
		dirty = true;
	} else if (r.bottom > mapRect.bottom) {
		SetCtlValue(mwg->mapVScroll, GetCtlValue(mwg->mapVScroll) + (r.bottom - mapRect.bottom));
		dirty = true;
	}	
	if (dirty) {
		SetClip(save);
		CalcAndDrawMapWindow(mwg->mapWindow);
		if (mwg->selectedPts.left != -1) {
			MPtoQDRect(mwg->mapWindow,&mwg->selectedPts,&r);
			InvertSelectedRect(mwg->mapWindow);
		}
		DrawControls(mwg->mapWindow);
		SetClip(clip);
	}		
	
	if (adjust) {
		// adjust cursor
		mwg->selectedPts.left++;
		if (mwg->selectedPts.left > (mpSize-1)) {
			mwg->selectedPts.left = 0;
			mwg->selectedPts.top++;
			if (mwg->selectedPts.top > (mpSize-1)) mwg->selectedPts.top = 0;
		}
		mwg->selectedPts.right=mwg->selectedPts.left;
		mwg->selectedPts.bottom=mwg->selectedPts.top;
		InvertSelectedRect(mwg->mapWindow);
	}

	SetClip(save);
	DisposeRgn(save);
	DisposeRgn(clip);
	HUnlock(gScenario.outsideMaps[mwg->mapID].map);
	mwg->dirty = true;
}

Boolean DoSelect(Point initialPt, Point mapPt, MapWindowData *mwg)
{
	Rect r,selMapPts,s;
	Pattern gray;

	gray.pat[0] = gray.pat[2] = gray.pat[4] = gray.pat[6] = 0xAA; // 10101010
	gray.pat[1] = gray.pat[3] = gray.pat[5] = gray.pat[7] = 0x55; // 01010101

	PenMode(patXor);//This makes it easy to fix the pixels
 	PenPat(&gray);

	selMapPts.left   = (initialPt.h < mapPt.h) ? initialPt.h : mapPt.h;
	selMapPts.top    = (initialPt.v < mapPt.v) ? initialPt.v : mapPt.v;
	selMapPts.right  = (initialPt.h > mapPt.h) ? initialPt.h : mapPt.h;
	selMapPts.bottom = (initialPt.v > mapPt.v) ? initialPt.v : mapPt.v;
	
	// calculate the area rect.
	MPtoQDRect(mwg->mapWindow,&selMapPts,&r);
	
	if (mwg->selectedPts.left != -1) {
		MPtoQDRect(mwg->mapWindow,&mwg->selectedPts,&s);
		FrameRect(&s);
	}
	FrameRect(&r);
	mwg->selectedPts = selMapPts;

	PenMode(patCopy);
	PenPat(&qd.black);
	
	return false;
}

// initial point is last point; so we can re-draw it.
Boolean DoDragMonster(Point initialPt, Point mapPt, MapWindowData *mwg)
{
	Rect r,s;
	long h,v;
	unsigned char *a;
	unsigned char former; 
	long mpSize = gScenario.outsideMapsSize[mwg->mapID].h; // mdt 3/11/96
	
	a = (unsigned char *)*gScenario.outsideMaps[mwg->mapID].map;
	
	// calculate last area
	h = initialPt.h;
	v = initialPt.v;
	s.top = (v * 32) - GetCtlValue(mwg->mapVScroll);
	s.left = (h * 32) - GetCtlValue(mwg->mapHScroll) + (kToolSmallOffset * 2) + kToolWidth;
	s.bottom = s.top + 32;
	s.right = s.left + 32;
	if (EqualPt(initialPt,mwg->monster.monsPosition)) {
		former = mwg->monster.monsTileOnTopOf;
	} else {
		former = a[(mpSize*v) + h];
	}

	// set map array coordinates
	h = mapPt.h;
	v = mapPt.v;
	
	// calculate the area rect.
	r.top = (v * 32) - GetCtlValue(mwg->mapVScroll);
	r.left = (h * 32) - GetCtlValue(mwg->mapHScroll) + (kToolSmallOffset * 2) + kToolWidth;
	r.bottom = r.top + 32;
	r.right = r.left + 32;

	// Set the map array
	//a[(64*v) + h] = (unsigned char)(mwg->tileType);

	// draw former tile.
	if ((mwg->monster.monsPosition.h == h) && (mwg->monster.monsPosition.v == v)) a[(64*initialPt.v) + initialPt.h] = former; // MDT 3/11/96
	PlotTile(gScenario.tiles,former,(GWorldPtr)(mwg->mapWindow),&s);
	
	// draw new tile.
	PlotTile(gScenario.tiles,(unsigned char)(mwg->monster.monsTileType),(GWorldPtr)(mwg->mapWindow),&r);
	
	return true; 
}

// initialPt unused.
Boolean DoPlotMonster(Point initialPt, Point mapPt, MapWindowData *mwg)
{
	Rect r,s;
	long h,v,x,y;
	unsigned char *a;
	long mpSize = gScenario.outsideMapsSize[mwg->mapID].h; // mdt 3/11/96
	
	a = (unsigned char *)*gScenario.outsideMaps[mwg->mapID].map;

	// set original location info.
	x = mwg->monster.monsPosition.h;
	y = mwg->monster.monsPosition.v;
	s.top = (y * 32) - GetCtlValue(mwg->mapVScroll);
	s.left = (x * 32) - GetCtlValue(mwg->mapHScroll) + (kToolSmallOffset * 2) + kToolWidth;
	s.bottom = s.top + 32;
	s.right = s.left + 32;

	// set new location info.
	h = mapPt.h;
	v = mapPt.v;
	r.top = (v * 32) - GetCtlValue(mwg->mapVScroll);
	r.left = (h * 32) - GetCtlValue(mwg->mapHScroll) + (kToolSmallOffset * 2) + kToolWidth;
	r.bottom = r.top + 32;
	r.right = r.left + 32;

	// Set the map array
	a[(mpSize*y) + x] = (unsigned char)(mwg->monster.monsTileOnTopOf);
	a[(mpSize*v) + h] = (unsigned char)(mwg->monster.monsTileType);

	// draw new tile.
	PlotTile(gScenario.tiles,(unsigned char)mwg->monster.monsTileOnTopOf,(GWorldPtr)(mwg->mapWindow),&s);
	PlotTile(gScenario.tiles,(unsigned char)mwg->monster.monsTileType,(GWorldPtr)(mwg->mapWindow),&r);
	
	// update monster information
	mwg->monster.monsPosition.h = h;
	mwg->monster.monsPosition.v = v;
	SetMonster(mwg->mapID,&mwg->monster);
	
	return true; // so we remember to recalculate.
}