/*
 * WatchCursor.c
 *
 * Watch Cursor Library for Color Cursors 3.0
 */

#include "Cursors30.h"
#include "WatchCursor.h"

// ACUR resource for watch cursor.
#define kWatchACUR		200

/* DoWatchCursor
 * Implements a simple spinning watch cursor.
 *   w		- included for backwards compatability.
 *	 action	- a constant, containing one of three values:
 *               kWatchOn   - turns watch cursor on.
 *               kWatchOff  - turns watch cursor off, freeing memory.
 *               kWatchSpin - increments the watch cursor by one frame.
 *            Note that a given action may be performed more than once without
 *            any unusual effects (save kWatchSpin, of course). kWatchSpin
 *            can be called w/o calling kWatchOn first.
 */
void DoWatchCursor(WindowPtr w, short action)
{
	static on = false;
	
	switch(action) {
	case kWatchOn:
		if (!on) {
			CCSimpleCursor(animation, kWatchACUR, true);
			on = true;
		}
		break;
	case kWatchOff:
		if (on) {
			CCSimpleCursor(animation, kWatchACUR, false);
			on = false;
		}
		break;
	case kWatchSpin:
		if (!on) {
			CCSimpleCursor(animation, kWatchACUR, true);
			on = true;
		}
		CCAnimateCursor(nil);
		break;
	}
}